@@ -1,10 +1,10 @@
-#define CATCH_CONFIG_MAIN
-#if __has_include(<catch2/catch.hpp>)
-#include <catch2/catch.hpp>
+#if __has_include(<catch2/catch_all.hpp>)
+#include <catch2/catch_all.hpp>
+#include <catch2/catch_test_macros.hpp>
 #else
 #include <catch.hpp>
 #endif
-#include "cdfpp/cdf-endianness.hpp"
+#include "cdfpp/cdf-io/endianness.hpp"
 #include <cstdint>
 
 
