@@ -2,13 +2,14 @@
 #include <cstdint>
 #include <optional>
 #include <string>
+#include <tuple>
 #include <unordered_map>
 #include <vector>
 
 
-#define CATCH_CONFIG_MAIN
-#if __has_include(<catch2/catch.hpp>)
-#include <catch2/catch.hpp>
+#if __has_include(<catch2/catch_all.hpp>)
+#include <catch2/catch_all.hpp>
+#include <catch2/catch_test_macros.hpp>
 #else
 #include <catch.hpp>
 #endif
@@ -56,37 +57,60 @@ std::enable_if_t<std::is_same_v<value_type, const char*>, bool> impl_compare_att
 }
 
 template <int index, typename value_type>
-std::enable_if_t<std::is_scalar_v<value_type> && !std::is_same_v<value_type, const char*>, bool>
-impl_compare_attribute_value(const cdf::Attribute& attribute, const value_type& value)
+std::enable_if_t<std::is_same_v<value_type, const char*>, bool> impl_compare_attribute_value(
+    const cdf::VariableAttribute& attribute, const value_type& value)
 {
-    return attribute.get<value_type>(index) == std::vector<value_type> { value };
+    auto attr_value = attribute.get<char>();
+    return std::string { attr_value.data(), std::size(attr_value) } == std::string { value };
 }
 
 template <int index, typename value_type>
-auto impl_compare_attribute_value(const cdf::Attribute& attribute, const value_type& value)
-    -> decltype(std::cbegin(value), value.at(0), true)
+std::enable_if_t<std::is_scalar_v<value_type> && !std::is_same_v<value_type, const char*>, bool>
+impl_compare_attribute_value(const cdf::Attribute& attribute, const value_type& value)
 {
-    return attribute
-               .get<typename std::remove_const_t<std::remove_reference_t<value_type>>::value_type>(
-                   index)
+    return attribute.get<value_type>(index) == no_init_vector<value_type> { value };
+}
+
+template <int index, typename value_type>
+std::enable_if_t<std::is_scalar_v<value_type> && !std::is_same_v<value_type, const char*>, bool>
+impl_compare_attribute_value(const cdf::VariableAttribute& attribute, const value_type& value)
+{
+    return attribute.get<value_type>() == no_init_vector<value_type> { value };
+}
+
+template <int index, typename attr_t, typename value_type>
+auto impl_compare_attribute_value(const attr_t& attribute,
+    const value_type& value) -> decltype(std::cbegin(value), value.at(0), attribute.value(), true)
+{
+    return attribute.template get<
+               typename std::remove_const_t<std::remove_reference_t<value_type>>::value_type>()
         == value;
 }
 
-template <int index, typename T>
-bool compare_attribute_value(const cdf::Attribute& attribute, const T& values)
+template <int index, typename attr_t, typename value_type>
+auto impl_compare_attribute_value(const attr_t& attribute,
+    const value_type& value) -> decltype(std::cbegin(value), value.at(0), attribute[0], true)
+{
+    return attribute.template get<
+               typename std::remove_const_t<std::remove_reference_t<value_type>>::value_type>(index)
+        == value;
+}
+
+template <int index, typename attr_t, typename T>
+bool compare_attribute_value(const attr_t& attribute, const T& values)
 {
     auto value = std::get<index>(values);
     return impl_compare_attribute_value<index>(attribute, value);
 }
 
-template <typename T, std::size_t... I>
-bool impl_compare_attribute_values(cdf::Attribute& attribute, T values, std::index_sequence<I...>)
+template <typename attr_t, typename T, std::size_t... I>
+bool impl_compare_attribute_values(attr_t& attribute, T values, std::index_sequence<I...>)
 {
     return (... && compare_attribute_value<I>(attribute, values));
 }
 
-template <typename... Ts>
-bool compare_attribute_values(cdf::Attribute& attribute, Ts... values)
+template <typename attr_t, typename... Ts>
+bool compare_attribute_values(attr_t& attribute, Ts... values)
 {
     return impl_compare_attribute_values(
         attribute, std::make_tuple(values...), std::make_index_sequence<sizeof...(values)> {});
@@ -94,13 +118,13 @@ bool compare_attribute_values(cdf::Attribute& attribute, Ts... values)
 
 bool compare_shape(const cdf::Variable& variable, std::initializer_list<uint32_t> values)
 {
-    return variable.shape() == std::vector<uint32_t> { values };
+    return variable.shape() == no_init_vector<uint32_t> { values };
 }
 
 
 template <typename value_type>
 bool check_variable(const cdf::Variable& var, std::initializer_list<uint32_t> expected_shape,
-    std::vector<value_type> ref)
+    no_init_vector<value_type> ref)
 {
     auto values = var.get<value_type>();
     bool is_valid = compare_shape(var, expected_shape);
@@ -131,11 +155,11 @@ bool check_time_variable(const cdf::Variable& var, std::initializer_list<uint32_
     auto offset = 0;
     if constexpr (std::is_same_v<time_t, cdf::tt2000_t>)
         offset = 5; // skip 1971 issue with tt2000
-    auto values = std::vector<decltype(cdf::to_time_point(time_t {}))>();
+    auto values = no_init_vector<decltype(cdf::to_time_point(time_t {}))>();
     std::transform(std::cbegin(var.get<time_t>()), std::cend(var.get<time_t>()),
         std::back_inserter(values), [](const time_t& v) { return cdf::to_time_point(v); });
     bool is_valid = compare_shape(var, expected_shape);
-    auto ref = std::vector<decltype(cdf::to_time_point(time_t {}))>(std::size(values));
+    auto ref = no_init_vector<decltype(cdf::to_time_point(time_t {}))>(std::size(values));
     std::generate(std::begin(ref), std::end(ref),
         [i = 0]() mutable { return time_point<system_clock> {} + hours(24 * (i++ * 180)) + 0ns; });
     is_valid &= std::inner_product(std::begin(ref) + offset, std::end(ref),
@@ -148,9 +172,9 @@ struct cos_gen
 {
     const T step;
     cos_gen(T step) : step { step } { }
-    std::vector<T> operator()(std::size_t size)
+    no_init_vector<T> operator()(std::size_t size)
     {
-        std::vector<T> values(size);
+        no_init_vector<T> values(size);
         std::generate(std::begin(values), std::end(values),
             [i = T(0.), step = step]() mutable
             {
@@ -165,9 +189,9 @@ struct cos_gen
 template <typename T>
 struct ones
 {
-    std::vector<T> operator()(std::size_t size)
+    no_init_vector<T> operator()(std::size_t size)
     {
-        std::vector<T> values(size);
+        no_init_vector<T> values(size);
         std::generate(std::begin(values), std::end(values), []() mutable { return T(1); });
         return values;
     }
@@ -176,9 +200,9 @@ struct ones
 template <typename T>
 struct zeros
 {
-    std::vector<T> operator()(std::size_t size)
+    no_init_vector<T> operator()(std::size_t size)
     {
-        std::vector<T> values(size);
+        no_init_vector<T> values(size);
         std::generate(std::begin(values), std::end(values), []() mutable { return T(0); });
         return values;
     }
@@ -194,27 +218,33 @@ std::size_t filesize(std::fstream& file)
     return static_cast<std::size_t>(pos);
 }
 
+#define CHECK_CDF_FILE(cd, expected_version, expected_majority, expected_compression)              \
+    REQUIRE(cd.distribution_version == expected_version);                                          \
+    REQUIRE(cd.majority == expected_majority);                                                     \
+    REQUIRE(cd.compression == expected_compression);
+
 
 #define CHECK_ATTRIBUTES(cd)                                                                       \
     REQUIRE(std::size(cd.attributes) == 8);                                                        \
     REQUIRE(has_attribute(cd, "attr"));                                                            \
     REQUIRE(compare_attribute_values(cd.attributes["attr"], "a cdf text attribute"));              \
     REQUIRE(has_attribute(cd, "attr_float"));                                                      \
-    REQUIRE(compare_attribute_values(cd.attributes["attr_float"], std::vector { 1.f, 2.f, 3.f },   \
-        std::vector { 4.f, 5.f, 6.f }));                                                           \
+    REQUIRE(compare_attribute_values(cd.attributes["attr_float"],                                  \
+        no_init_vector<float> { 1.f, 2.f, 3.f }, no_init_vector<float> { 4.f, 5.f, 6.f }));        \
     REQUIRE(has_attribute(cd, "attr_int"));                                                        \
-    REQUIRE(compare_attribute_values(                                                              \
-        cd.attributes["attr_int"], std::vector { int8_t { 1 }, int8_t { 2 }, int8_t { 3 } }));     \
+    REQUIRE(compare_attribute_values(cd.attributes["attr_int"],                                    \
+        no_init_vector<int8_t> { int8_t { 1 }, int8_t { 2 }, int8_t { 3 } }));                     \
     REQUIRE(has_attribute(cd, "attr_multi"));                                                      \
     REQUIRE(compare_attribute_values(cd.attributes["attr_multi"],                                  \
-        std::vector { int8_t { 1 }, int8_t { 2 } }, std::vector { 2.f, 3.f }, "hello"));           \
+        no_init_vector<int8_t> { int8_t { 1 }, int8_t { 2 } }, no_init_vector<float> { 2.f, 3.f }, \
+        "hello"));                                                                                 \
     REQUIRE(has_attribute(cd, "empty"));                                                           \
     REQUIRE(std::size(cd.attributes["empty"]) == 0UL);                                             \
     REQUIRE(has_attribute(cd, "epoch"))
 
 
 #define CHECK_VARIABLES(cd)                                                                        \
-    REQUIRE(std::size(cd.variables) == 16);                                                        \
+    REQUIRE(std::size(cd.variables) == 18);                                                        \
     REQUIRE(has_variable(cd, "var"));                                                              \
     REQUIRE(compare_shape(cd.variables["var"], { 101 }));                                          \
     REQUIRE(check_variable(                                                                        \
@@ -241,26 +271,26 @@ std::size_t filesize(std::fstream& file)
     REQUIRE(has_variable(cd, "var3d"));                                                            \
     REQUIRE(compare_shape(cd.variables["var3d"], { 4, 3, 2 }));                                    \
     REQUIRE(check_variable(cd.variables["var3d"], { 4, 3, 2 }, ones<double>()));                   \
-    REQUIRE(compare_attribute_values(                                                              \
-        cd.variables["var3d"].attributes["var3d_attr_multi"], std::vector { 10., 11. }));          \
+    REQUIRE(compare_attribute_values(cd.variables["var3d"].attributes["var3d_attr_multi"],         \
+        no_init_vector<double> { 10., 11. }));                                                     \
     REQUIRE(has_variable(cd, "var2d_counter"));                                                    \
     REQUIRE(compare_shape(cd.variables["var2d_counter"], { 10, 10 }));                             \
     REQUIRE(check_variable(cd.variables["var2d_counter"], { 10, 10 },                              \
         [](const auto&)                                                                            \
         {                                                                                          \
-            std::vector<double> result(100);                                                       \
+            no_init_vector<double> result(100);                                                    \
             std::iota(std::begin(result), std::end(result), 0.);                                   \
             return result;                                                                         \
         }));                                                                                       \
-    REQUIRE(check_variable(cd.variables["var_string"], { 16 },                                     \
-        std::vector<char> {                                                                        \
+    REQUIRE(check_variable(cd.variables["var_string"], { 1, 16 },                                  \
+        no_init_vector<char> {                                                                     \
             'T', 'h', 'i', 's', ' ', 'i', 's', ' ', 'a', ' ', 's', 't', 'r', 'i', 'n', 'g' }));    \
-    REQUIRE(check_variable(cd.variables["var_string_uchar"], { 16 },                               \
-        std::vector<unsigned char> {                                                               \
+    REQUIRE(check_variable(cd.variables["var_string_uchar"], { 1, 16 },                            \
+        no_init_vector<unsigned char> {                                                            \
             'T', 'h', 'i', 's', ' ', 'i', 's', ' ', 'a', ' ', 's', 't', 'r', 'i', 'n', 'g' }))
 
 
-SCENARIO("Loading a cdf files", "[CDF]")
+SCENARIO("Loading cdf files", "[CDF]")
 {
     GIVEN("a cdf file")
     {
@@ -280,13 +310,20 @@ SCENARIO("Loading a cdf files", "[CDF]")
                 REQUIRE(cdf::io::load(path) == std::nullopt);
             }
         }
-        WHEN("file exists and is a cdf file")
+        WHEN("file exists and is an uncompressed cdf file")
         {
             auto path = std::string(DATA_PATH) + "/a_cdf.cdf";
             REQUIRE(file_exists(path));
             auto cd_opt = cdf::io::load(path);
             REQUIRE(cd_opt != std::nullopt);
             auto cd = *cd_opt;
+            THEN("Version, majority and compression type are retrieved")
+            {
+                static constexpr auto version
+                    = std::tuple<uint32_t, uint32_t, uint32_t> { 3, 9, 0 };
+                CHECK_CDF_FILE(
+                    cd, version, cdf::cdf_majority::row, cdf::cdf_compression_type::no_compression);
+            }
             THEN("All expected attributes are loaded")
             {
                 CHECK_ATTRIBUTES(cd);
@@ -314,6 +351,13 @@ SCENARIO("Loading a cdf files", "[CDF]")
             auto cd_opt = cdf::io::load(data);
             REQUIRE(cd_opt != std::nullopt);
             auto cd = *cd_opt;
+            THEN("Version and majority are retrieved")
+            {
+                static constexpr auto version
+                    = std::tuple<uint32_t, uint32_t, uint32_t> { 3, 9, 0 };
+                CHECK_CDF_FILE(
+                    cd, version, cdf::cdf_majority::row, cdf::cdf_compression_type::no_compression);
+            }
             THEN("All expected attributes are loaded")
             {
                 CHECK_ATTRIBUTES(cd);
@@ -327,29 +371,73 @@ SCENARIO("Loading a cdf files", "[CDF]")
         {
             auto path = std::string(DATA_PATH) + "/a_cdf.cdf";
             REQUIRE(file_exists(path));
-            auto [data, size] = [&]()
+            THEN("It can be eagerly loaded")
             {
-                std::fstream file { path, std::ios::binary | std::ios::in };
-                if (file.is_open())
+                auto [data, size] = [&]()
                 {
-                    std::size_t size = filesize(file);
-                    char* data = new char[size];
-                    file.read(data, static_cast<int64_t>(size));
-                    return std::make_tuple(data, size);
+                    std::fstream file { path, std::ios::binary | std::ios::in };
+                    if (file.is_open())
+                    {
+                        std::size_t size = filesize(file);
+                        char* data = new char[size];
+                        file.read(data, static_cast<int64_t>(size));
+                        return std::make_tuple(data, size);
+                    }
+                    return std::make_tuple(static_cast<char*>(nullptr), std::size_t { 0UL });
+                }();
+                auto cd_opt = cdf::io::load(data, size);
+                delete[] data;
+                REQUIRE(cd_opt != std::nullopt);
+                auto cd = *cd_opt;
+                THEN("Version and majority are retrieved")
+                {
+                    static constexpr auto version
+                        = std::tuple<uint32_t, uint32_t, uint32_t> { 3, 9, 0 };
+                    CHECK_CDF_FILE(cd, version, cdf::cdf_majority::row,
+                        cdf::cdf_compression_type::no_compression);
+                }
+                THEN("All expected attributes are loaded")
+                {
+                    CHECK_ATTRIBUTES(cd);
+                }
+                THEN("All expected variables are loaded")
+                {
+                    CHECK_VARIABLES(cd);
                 }
-                return std::make_tuple(static_cast<char*>(nullptr), std::size_t { 0UL });
-            }();
-            auto cd_opt = cdf::io::load(data, size);
-            delete[] data;
-            REQUIRE(cd_opt != std::nullopt);
-            auto cd = *cd_opt;
-            THEN("All expected attributes are loaded")
-            {
-                CHECK_ATTRIBUTES(cd);
             }
-            THEN("All expected variables are loaded")
+            THEN("It can be lazy loaded")
             {
-                CHECK_VARIABLES(cd);
+                auto [data, size] = [&]()
+                {
+                    std::fstream file { path, std::ios::binary | std::ios::in };
+                    if (file.is_open())
+                    {
+                        std::size_t size = filesize(file);
+                        char* data = new char[size];
+                        file.read(data, static_cast<int64_t>(size));
+                        return std::make_tuple(data, size);
+                    }
+                    return std::make_tuple(static_cast<char*>(nullptr), std::size_t { 0UL });
+                }();
+                auto cd_opt = cdf::io::load(data, size, true);
+                REQUIRE(cd_opt != std::nullopt);
+                auto cd = *cd_opt;
+                THEN("Version and majority are retrieved")
+                {
+                    static constexpr auto version
+                        = std::tuple<uint32_t, uint32_t, uint32_t> { 3, 9, 0 };
+                    CHECK_CDF_FILE(cd, version, cdf::cdf_majority::row,
+                        cdf::cdf_compression_type::no_compression);
+                }
+                THEN("All expected attributes are loaded")
+                {
+                    CHECK_ATTRIBUTES(cd);
+                }
+                THEN("All expected variables are loaded")
+                {
+                    CHECK_VARIABLES(cd);
+                }
+                delete[] data;
             }
         }
         WHEN("file exists and is a compressed cdf file (GZIP)")
@@ -359,6 +447,13 @@ SCENARIO("Loading a cdf files", "[CDF]")
             auto cd_opt = cdf::io::load(path);
             REQUIRE(cd_opt != std::nullopt);
             auto cd = *cd_opt;
+            THEN("Version and majority are retrieved")
+            {
+                static constexpr auto version
+                    = std::tuple<uint32_t, uint32_t, uint32_t> { 3, 9, 0 };
+                CHECK_CDF_FILE(cd, version, cdf::cdf_majority::row,
+                    cdf::cdf_compression_type::gzip_compression);
+            }
             THEN("All expected attributes are loaded")
             {
                 CHECK_ATTRIBUTES(cd);
@@ -375,6 +470,13 @@ SCENARIO("Loading a cdf files", "[CDF]")
             auto cd_opt = cdf::io::load(path);
             REQUIRE(cd_opt != std::nullopt);
             auto cd = *cd_opt;
+            THEN("Version and majority are retrieved")
+            {
+                static constexpr auto version
+                    = std::tuple<uint32_t, uint32_t, uint32_t> { 3, 9, 0 };
+                CHECK_CDF_FILE(cd, version, cdf::cdf_majority::row,
+                    cdf::cdf_compression_type::rle_compression);
+            }
             THEN("All expected attributes are loaded")
             {
                 CHECK_ATTRIBUTES(cd);
@@ -391,6 +493,13 @@ SCENARIO("Loading a cdf files", "[CDF]")
             auto cd_opt = cdf::io::load(path);
             REQUIRE(cd_opt != std::nullopt);
             auto cd = *cd_opt;
+            THEN("Version and majority are retrieved")
+            {
+                static constexpr auto version
+                    = std::tuple<uint32_t, uint32_t, uint32_t> { 3, 9, 0 };
+                CHECK_CDF_FILE(
+                    cd, version, cdf::cdf_majority::row, cdf::cdf_compression_type::no_compression);
+            }
             THEN("All expected attributes are loaded")
             {
                 CHECK_ATTRIBUTES(cd);
@@ -407,6 +516,13 @@ SCENARIO("Loading a cdf files", "[CDF]")
             auto cd_opt = cdf::io::load(path);
             REQUIRE(cd_opt != std::nullopt);
             auto cd = *cd_opt;
+            THEN("Version and majority are retrieved")
+            {
+                static constexpr auto version
+                    = std::tuple<uint32_t, uint32_t, uint32_t> { 3, 9, 0 };
+                CHECK_CDF_FILE(cd, version, cdf::cdf_majority::column,
+                    cdf::cdf_compression_type::no_compression);
+            }
             THEN("All expected attributes are loaded")
             {
                 CHECK_ATTRIBUTES(cd);
@@ -416,5 +532,41 @@ SCENARIO("Loading a cdf files", "[CDF]")
                 CHECK_VARIABLES(cd);
             }
         }
+        WHEN("file is a 2.4.x cdf")
+        {
+            auto path = std::string(DATA_PATH) + "/ia_k0_epi_19970102_v01.cdf";
+            REQUIRE(file_exists(path));
+            auto cd_opt = cdf::io::load(path);
+            REQUIRE(cd_opt != std::nullopt);
+            auto cd = *cd_opt;
+            THEN("All expected variables are found")
+            {
+                REQUIRE(std::size(cd.variables) == 10);
+            }
+        }
+        WHEN("file is a 2.4.x cdf")
+        {
+            auto path = std::string(DATA_PATH) + "/ge_k0_cpi_19921231_v02.cdf";
+            REQUIRE(file_exists(path));
+            auto cd_opt = cdf::io::load(path);
+            REQUIRE(cd_opt != std::nullopt);
+            auto cd = *cd_opt;
+            THEN("All expected variables are found")
+            {
+                REQUIRE(std::size(cd.variables) == 25);
+            }
+        }
+        WHEN("file is a 2.5.x cdf")
+        {
+            auto path = std::string(DATA_PATH) + "/ac_h2_sis_20101105_v06.cdf";
+            REQUIRE(file_exists(path));
+            auto cd_opt = cdf::io::load(path);
+            REQUIRE(cd_opt != std::nullopt);
+            auto cd = *cd_opt;
+            THEN("All expected variables are found")
+            {
+                REQUIRE(std::size(cd.variables) == 61);
+            }
+        }
     }
 }
